/**
 * rollup-plugin-resolve2 v1.0.0
 *
 * @author Allex Wang <allex.wxn@gmail.com>
 * Released under the MIT License.
 */
var path = require('path');
var fs = require('fs');

var localImport = /^[.]{1,2}\//;

var isAlias = function isAlias(file, alias) {
  if (alias === file) {
    return true;
  }
  if (!file.startsWith(alias)) {
    return false;
  }
  return file[alias.length] === '/';
};

var getAlias = function getAlias(file, aliases) {
  for (var p in aliases) {
    if (aliases.hasOwnProperty(p) && isAlias(file, p)) return p;
  }
  return null;
};

var getExistingFileWithExt = function getExistingFileWithExt(file, extensions) {
  for (var i = 0; i < extensions.length; i++) {
    var f = file + extensions[i];
    if (fs.existsSync(f)) {
      return f;
    }
  }
  return null;
};

var resolveModuleEntry = function resolveModuleEntry(dir, options) {
  var pkgfile = path.join(dir, 'package.json'),
      pkg = void 0;
  if (fs.existsSync(pkgfile) && (pkg = require(pkgfile))) {
    var mainFields = options.mainFields.reduce(function (p, mainField) {
      var t = pkg[mainField];
      if (t && p.indexOf(t) === -1) p.push(t);
      return p;
    }, []);

    var main = mainFields.reduce(function (p, mainField) {
      if (!p) {
        var f = path.join(dir, mainField);
        if (fs.existsSync(f) && fs.statSync(f).isFile()) {
          return f;
        }
      }
      return p;
    }, '') || path.join(dir, 'index.js');
    return main;
  }
};

module.exports = function (options) {
  if (!options) {
    options = { extensions: ['.js'], alias: {} };
  } else {
    if (Array.isArray(options)) {
      options = { extensions: options };
    }
    if (!options.extensions) {
      options.extensions = ['.js'];
    }
    if (!options.alias) {
      options.alias = {};
    }
  }

  if (options.extensions.length === 0 && Object.keys(options.alias).length === 0) {
    return {};
  }

  if (!options.indexFile) {
    options.indexFile = 'index';
  }

  options.mainFields = options.mainFields || ['module', 'main', 'jsnext:main', 'browser'];

  var resolvedCache = {};

  return {
    resolveId(importee, importer) {
      if (!importer) {
        return null;
      }

      var file = null;
      if (!localImport.test(importee)) {
        var alias = getAlias(importee, options.alias);
        if (alias === null) {
          return null;
        }
        file = importee.substr(alias.length);
        if (file !== '') {
          file = '.' + file;
        }
        file = path.resolve(options.alias[alias], file);
      } else {
        file = path.resolve(importer, '..', importee);
      }

      if (resolvedCache.hasOwnProperty(file)) {
        return resolvedCache[file];
      }

      var key = file;
      var addCache = function addCache(v) {
        if (v) resolvedCache[key] = v;
        return v;
      };

      if (fs.existsSync(file)) {
        if (!fs.statSync(file).isDirectory()) {
          return addCache(file);
        } else {
          var entry = resolveModuleEntry(file, options);
          if (entry) {
            return addCache(entry);
          }
        }

        var f = getExistingFileWithExt(file, options.extensions);
        if (f !== null) {
          return addCache(f);
        }

        file = path.resolve(file, '.', options.indexFile);
      }

      return addCache(getExistingFileWithExt(file, options.extensions));
    }
  };
};
